/*
 * translatedialog.cpp
 *
 * Copyright (c) 2005 Michael Rex <me@rexi.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "translatedialog.h"
#include "kdingconfig.h"
#include <kcombobox.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <kapplication.h>
#include <khtmlview.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kconfig.h>
#include <khtml_part.h>
#include <kanimwidget.h>
#include <kstatusbar.h>
//#include <kcompletion.h>
//#include <kglobal.h>
#include <qregexp.h>
#include <qstringlist.h>
#include <qlayout.h>
#include <qcolor.h>
#include <qtooltip.h>

TranslateDialog::TranslateDialog(QWidget* parent, const char* name) : TranslateDialogBase(parent, name), dict(this) {
    setIcon(kapp->miniIcon());
    
    // fill the combobox with the recently used terms
    histText->setHistoryItems(KDingConfig::self()->recent(), true);
    
    connect(histText, SIGNAL(cleared()), this, SLOT(historyCleared()));
    
    // connect DingDictionary's signals to our slots
    connect(&dict, SIGNAL(searchRunning()), this, SLOT(searchAlreadyRunning()));
    connect(&dict, SIGNAL(searchFinished()), this, SLOT(fetchResults()));
    
    animSearch = new KAnimWidget("kding_search", 22, this);
    layoutTopLine->addWidget(animSearch);
    
    // create and add KHTMLPart and display "Welcome" page
    htmlpart = new KHTMLPart(this);
    TranslateDialogBaseLayout->addWidget(htmlpart->view());
    htmlpart->setJScriptEnabled(false);
    htmlpart->setJavaEnabled(false);
    htmlpart->setMetaRefreshEnabled(false);
    htmlpart->setPluginsEnabled(false);
    htmlpart->openURL(getWelcomeLocation());
    
    statusBar = new KStatusBar(this);
    TranslateDialogBaseLayout->addWidget(statusBar);
    statusBar->insertItem(i18n("Dictionary version %1").arg(dict.dictionaryVersion()), SB_MESSAGE);
}

TranslateDialog::~TranslateDialog() {
    // store history of search terms
    QStringList list = histText->historyItems();
    KDingConfig::self()->setRecent(list);
    KDingConfig::self()->writeConfig();
}

void TranslateDialog::translateClicked() {
    QString text = histText->currentText();
    translate(text);
}

void TranslateDialog::historyCleared() {
    QStringList list = histText->historyItems();
    KDingConfig::self()->setRecent(list);
    KDingConfig::self()->writeConfig();
}

/**
 * 
 */
void TranslateDialog::translate(const QString& text) {
    /* do some sanity checks first: */
    // no empty search term
    if(text.isEmpty()) {
        return;
    }
    
    QString searchText = text.stripWhiteSpace();
    
    // go for it
    animSearch->start();
    histText->setCurrentText(searchText);
    statusBar->changeItem(i18n("Searching for '%1'").arg(searchText), SB_MESSAGE);
    
    dict.search(searchText);
}

void TranslateDialog::searchAlreadyRunning() {
    //KMessageBox::sorry(this, i18n("Another search is currently running. Please wait for it to finish before you start a new one."));
    statusBar->changeItem(i18n("Another search is currently running"), SB_MESSAGE);
}

void TranslateDialog::clearDialog() {
    histText->clearEdit();
    histText->reset();
    
    htmlpart->begin();
    htmlpart->write("<html><body></body></html>");
    htmlpart->end();
    
    //QToolTip::remove(animSearch);
    
    statusBar->changeItem("", SB_MESSAGE);
}

void TranslateDialog::resizeEvent(QResizeEvent* e) {
    QDialog::resizeEvent(e);
    
//    QHeader* header = listResults->header();
//    header->resizeSection(0, listResults->width() / 2);
//    header->resizeSection(1, listResults->width() / 2);
//    listResults->triggerUpdate();
}


KURL TranslateDialog::getCSSLocation() {
    QString path = locate("data", "kding/kding.css");
    KURL url = KURL::fromPathOrURL(path);
//    kdDebug() << path << " >> " << url << endl;
    return url;
}

KURL TranslateDialog::getWelcomeLocation() {
    QString path = locate("data", "kding/welcome.html");
    KURL url = KURL::fromPathOrURL(path);
//    kdDebug() << path << " >> " << url << endl;
    return url;
}

void TranslateDialog::fetchResults() {
    animSearch->stop();
    ResultList list = dict.results();
    kdDebug() << "Matching lines: " << list.size() << endl;
    
    //clearDialog();
    //histText->setEditText(dict.searchTerm());
    
    QString msgStr = i18n("Found 1 match for '%1'", 
                          "Found %n matches for '%1'", list.size()).arg(dict.searchTerm());
    //QToolTip::add(animSearch, msgStr);
    
    statusBar->changeItem(msgStr, SB_MESSAGE);
    
    /* And now, what to do if the search succeeded? Or failed, respectively */
    if(!list.isEmpty()) { /* search succeeded */
        this->show();
        
        // display results
        htmlpart->begin();
        htmlpart->setUserStyleSheet(getCSSLocation());
        htmlpart->write(getDocument(list, dict.searchTerm()));
        htmlpart->end();
        
        // save search term in history
        histText->addToHistory(dict.searchTerm());
        QStringList list = histText->historyItems();
        KDingConfig::self()->setRecent(list);
        KDingConfig::self()->writeConfig();
        
        //labelResultCount->setText(i18n("%1 results").arg(listResults->childCount()));
    } else { /* search failed */
        // set the color of the search term to red
        //histText->setPaletteForegroundColor(FAILED_COLOR);
        //histText->update();
        
        //labelResultCount->setText(i18n("No results for %1").arg(text));
        htmlpart->begin();
        htmlpart->setUserStyleSheet(getCSSLocation());
        htmlpart->write("<html><body><table><tr><td class=\"error\">No results</td></tr></table></body></html>");
        htmlpart->end();
    }
    
    // select the text in the lineedit
    QLineEdit* lineEdit = histText->lineEdit();
    lineEdit->setSelection(0, lineEdit->text().length());
}

void TranslateDialog::textChanged(const QString& str) {
    //histText->setPaletteForegroundColor(DEFAULT_COLOR);
}

QString TranslateDialog::getDocument(const ResultList& results, const QString& term) {
    QString html = "<html><body><table>";
    html += "<tr><th width=\"50%\">";
    html += i18n("table caption", "German");
    html += "</th><th width=\"50%\">";
    html += i18n("table caption", "English");
    html += "</th></tr>";
    
    // some RegExp used to format the output
    QRegExp re_split("(.*) :: (.*)");   // capture left and right side in groups
    QRegExp openbraces(" ([[{])");  // match opening braces: { [
    QRegExp closebraces("([]}])");  // match closing braces: } ]
    QRegExp searchterm("(" + term + ")", false);    // match the search term, regardless of case

    int bgclass = 0; // counter used for alternating background colors
    ResultList::const_iterator end = results.constEnd();
    for(ResultList::const_iterator it = results.constBegin(); it != end; ++it) {
        QString line = (*it).text();
        
        line.replace(searchterm, "<span class=\"keyword\">\\1</span>");
        line.replace("|", "<br>&nbsp;&nbsp;");
        line.replace(openbraces, "&nbsp;<i>\\1");
        line.replace(closebraces, "\\1</i>");
        re_split.search(line);
        QString ger = re_split.cap(1).stripWhiteSpace();
        QString eng = re_split.cap(2).stripWhiteSpace();

        html += "<tr><td class=\"c" + QString::number(bgclass % 2)
                + "\">" + ger + "</td><td class=\"c"
                + QString::number(bgclass % 2)
                + "\">" + eng + "</td></tr>";
        ++bgclass;
    }
    
    html += "</table></body></html>";
    
    return html;
}

